/* IBM Confidential
 * OCO Source Materials
 * 5724-L79
 *  Copyright IBM Corporation 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.v602_to_v610;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.provisional.core.IMessage;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ibm.j2ca.migration.MigrationContext;
import com.ibm.j2ca.migration.ValidationTask;
import com.ibm.j2ca.migration.util.SearchHelper;
import com.ibm.j2ca.migration.util.Util;

public class SAPAdapterNameValidationTask extends ValidationTask {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2007."; //$NON-NLS-1$
	private static final String ADAPTER_NAMEERROR_ID_KEY = "SAPAdapterNameValidationTask_AdapterNameError_Id";
	private static final String ADAPTER_NAMEERROR_MESSAGE_KEY = "SAPAdapterNameValidationTask_AdapterNameError_Message";
	private static final String QUICKFIX_CLASS = "com.ibm.j2ca.migration.sap.v602_to_v610.SAPAdapterNameFix";
	
	public ArrayList<Message> test(IProject project) throws Exception {
		return test(project, new Hashtable<String, String>());
	}
	
	public ArrayList<Message> test(IProject project, Hashtable<String, String> parameters) throws Exception {
		ArrayList<Message> messages = new ArrayList<Message>();
		
		MigrationContext migrationContext = new MigrationContext(project);		
		String sourceAdapterName = migrationContext.getSourceAdapterName();
			
		ArrayList<IFile> serviceDescriptorFiles = SearchHelper.getContainerFiles(project, getArtifactRegEx(), "");
		for(IFile file : serviceDescriptorFiles) {
			DOMParser parser = new DOMParser();				
			parser.parse(file.getLocation().toString());				
			Document doc = parser.getDocument();
			
			Element eResourceAdapter = (Element)doc.getElementsByTagName("resourceAdapter").item(0);
			if(eResourceAdapter!=null) {
				String adapterName = eResourceAdapter.getAttribute("name");
				
				if(adapterName.matches(".*\\.IBM WebSphere Adapter for SAP Software") && !adapterName.matches(".*\\." + sourceAdapterName)) {
					String serviceName = file.getName().replaceFirst("(\\.import)|(\\.export)", "");
					
					Message message = new Message();
					message.setBundleName(getMessageBundleName());
					message.setId(ADAPTER_NAMEERROR_MESSAGE_KEY);
					message.setSeverity(IMessage.HIGH_SEVERITY);
					message.setTargetObject(file);
					message.setParams(new String [] {serviceName});
					
					ResourceBundle resourceBundle = ResourceBundle.getBundle(getMessageBundleName(), Locale.getDefault(), this.getClass().getClassLoader());
					String flagId = Util.getBundleString(ADAPTER_NAMEERROR_ID_KEY, resourceBundle);
					message.setAttribute("id", flagId);
					message.setAttribute("resourceBundle", resourceBundle);
					message.setAttribute("quickFixClass", QUICKFIX_CLASS);
					
					messages.add(message);
				}
			}
		}
		
		return messages;
	}
	
	public String getArtifactRegEx() { return SERVICE_DESCRIPTOR_REGEX; }
	public String getMessageBundleName() { return MigrationMessages.BUNDLE_NAME; }

}
