/* IBM Confidential
 * OCO Source Materials
 * 5724-L79
 *  Copyright IBM Corporation 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.v602_to_v620;

import java.util.ArrayList;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Path;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ibm.j2ca.migration.ConnectorMigrationTask;
import com.ibm.j2ca.migration.MigrationContext;
import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.changedata.*;
import com.ibm.j2ca.migration.util.XMLUtil;

public class SAPConnectorMigrationTask extends ConnectorMigrationTask {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2007."; //$NON-NLS-1$
	
	public ArrayList<ConnectorChangeData> getChangeData() throws MigrationException {
		ArrayList<ConnectorChangeData> changeDataList = new ArrayList<ConnectorChangeData>();
		
		try {
			if(hasLocalTransaction(getMigrationContext())) {
				changeDataList.add( new UpdateRAR("IBM WebSphere Adapter for SAP Software with transaction support") );
			}
			else {
				changeDataList.add( new UpdateRAR() );
			}
		}
		catch(Exception e) {
			changeDataList.add( new UpdateRAR() );
		}
		
		
		changeDataList.add( new AddConnectorProjectLibrary("connectorModule/DESPI.jar") );
		changeDataList.add( new AddConnectorProjectLibrary("connectorModule/commonj.connector.jar") );
		
		return changeDataList;
	}
	
	public static boolean hasLocalTransaction(MigrationContext migrationContext) throws Exception {
		IProject connectorProject = migrationContext.getConnectorProject();
		IFile file = connectorProject.getFile(new Path("connectorModule/META-INF/ra.xml"));
		
		DOMParser parser = new DOMParser();				
		parser.parse(file.getLocation().toString());				
		Document doc = parser.getDocument();
		
		Element eTransactionSupport = (Element)doc.getElementsByTagName("transaction-support").item(0);
		if(eTransactionSupport!=null) {
			String transactionSupport = XMLUtil.getNodeText(eTransactionSupport);
			if(transactionSupport.equals("LocalTransaction")) {
				return true;
			}
		}
		
		return false;
	}
}
