/* IBM Confidential
 * OCO Source Materials
 * 5724-L79
 *  Copyright IBM Corporation 2007
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.v602_to_v620;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.provisional.core.IMessage;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.ibm.j2ca.migration.MigrationContext;
import com.ibm.j2ca.migration.ValidationTask;
import com.ibm.j2ca.migration.plugin.UpdateContributionDescriptor;
import com.ibm.j2ca.migration.plugin.UpdateRegistry;
import com.ibm.j2ca.migration.util.SearchHelper;
import com.ibm.j2ca.migration.util.Util;

public class SAPFunctionSelectorValidationTask  extends ValidationTask {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2007."; //$NON-NLS-1$
	public static final String MESSAGES_BUNDLE_NAME = "com.ibm.j2ca.migration.sap.v602_to_v610.messages"; //$NON-NLS-1$
	private static final String FUNCTIONSELECTOR_ERROR_ID_KEY = "SAPFunctionSelectorValidationTask_FunctionSelectorError_Id";
	private static final String FUNCTIONSELECTOR_ERROR_MESSAGE_KEY = "SAPFunctionSelectorValidationTask_FunctionSelectorError_Message";
	private static final String QUICKFIX_CLASS = "com.ibm.j2ca.migration.sap.v602_to_v610.SAPFunctionSelectorFix";
	
	public String getMessageBundleName() { return MESSAGES_BUNDLE_NAME; }
	
	public ArrayList<Message> test(IProject project) throws Exception {
		return test(project, new Hashtable<String, String>());
	}
	
	public ArrayList<Message> test(IProject project, Hashtable<String, String> parameters) throws Exception {
		ArrayList<Message> messages = new ArrayList<Message>();
		
		MigrationContext migrationContext = new MigrationContext(project);
		UpdateContributionDescriptor descriptor = UpdateRegistry.getInstance().getUpdateDescriptorByAdapterId(migrationContext.getTargetVersion().toValueString(), migrationContext.getSourceAdapterId());
		
		if(migrationContext.getSourceAdapterVersion().matches(descriptor.getAdapterInfo().getVersion()) ) {
			ArrayList<IFile> serviceDescriptorFiles = SearchHelper.getContainerFiles(project, getArtifactRegEx(), "");
			for(IFile file : serviceDescriptorFiles) {
				if(!hasValidFunctionSelector(file)) {
					String serviceName = file.getName().replaceFirst("(\\.import)|(\\.export)", "");
					
					Message message = new Message();
					message.setBundleName(getMessageBundleName());
					message.setId(FUNCTIONSELECTOR_ERROR_MESSAGE_KEY);
					message.setSeverity(IMessage.HIGH_SEVERITY);
					message.setTargetObject(file);
					message.setParams(new String [] {serviceName});
					
					ResourceBundle resourceBundle = ResourceBundle.getBundle(getMessageBundleName(), Locale.getDefault(), this.getClass().getClassLoader());
					String flagId = Util.getBundleString(FUNCTIONSELECTOR_ERROR_ID_KEY, resourceBundle);
					message.setAttribute("id", flagId);
					message.setAttribute("resourceBundle", resourceBundle);
					message.setAttribute("quickFixClass", QUICKFIX_CLASS);
					
					messages.add(message);
				}
			}
		}
		
		return messages;
	}
	
	private boolean hasValidFunctionSelector(IFile file) throws Exception {
		DOMParser parser = new DOMParser();
		parser.parse(file.getLocation().toString());				
		Document doc = parser.getDocument();
		
		NodeList nlRFCProgramID = doc.getElementsByTagName("rfcProgramID");
		Element eConnection = (Element)doc.getElementsByTagName("connection").item(0);
		if(eConnection!=null) {
			String selectorType = eConnection.getAttribute("selectorType");
			if(nlRFCProgramID.getLength()>0
					&& !selectorType.equals("com.ibm.j2ca.sap.emd.runtime.SAPFunctionSelectorImpl")) {
				return false;
			}
		}
		
		return true;
	}
	
	public String getArtifactRegEx() { return SERVICE_DESCRIPTOR_REGEX; }
}
