/* IBM Confidential
 * OCO Source Materials
 * 5724-N40, 5724-N41, 5724-N42, 5724-N43, 5724-L77, 5724-L78, 5724-L79, 5724-L80, 5724-L81, 5724-L82
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.wbi_to_v610;

import java.io.IOException;
import java.util.ArrayList;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.j2ca.migration.ArtifactChangeArguments;
import com.ibm.j2ca.migration.IChangeArguments;
import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.internal.changes.wbi.ConvertWBIASItoJCAChange;
import com.ibm.j2ca.migration.sap.v602_to_v610.MigrationMessages;
import com.ibm.j2ca.migration.util.AttributeMap;
import com.ibm.j2ca.migration.util.SearchHelper;

public class ConvertWBIASItoJCAASIForSAPChange extends ConvertWBIASItoJCAChange {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$
	
	private IFile file;
	protected ArrayList<IFile> topLevelBOs = null;
	private String BAPI_RS_FK_Value = null;
	
	public ConvertWBIASItoJCAASIForSAPChange(IFile file, ConvertWBIASItoJCAASIForSAP changeData) {
		super(changeData);
		this.file = file;
	}
	
	public ConvertWBIASItoJCAASIForSAP getChangeData() { return (ConvertWBIASItoJCAASIForSAP)super.getChangeData(); }
	
	public String getChangeDetails() {
		return MigrationMessages.applyParameters(MigrationMessages.ConvertWBIASItoJCAASIForSAPChange_Description,
				new String[] {getChangeData().namespacePrefix, 
					getChangeData().namespaceURI, 
					getChangeData().boMetadataNodeName,
					getChangeData().attributeMetadataNodeName});
	}
	
	public IChangeArguments getChangeArguments() {
		ArtifactChangeArguments argument = new ArtifactChangeArguments(file);
		return argument;
	}
	
	public void perform(IProgressMonitor pm) throws Exception {
		if(file.exists()) {
			DOMParser parser = new DOMParser();
			parser.parse(file.getLocation().toString());			
			Document doc = parser.getDocument();
			
			//modify BO definitions for BAPI - the info is from BG
			String bgFileLocation = file.getLocation().toString().substring(0, file.getLocation().toString().lastIndexOf(".")) + "BG.xsd";
			IFile bgFile = SearchHelper.getFile(file.getProject(), bgFileLocation);
			if(getChangeData().sapAPI.equalsIgnoreCase("BAPI") && bgFile != null && bgFile.exists() && isTopLevelBOBG(file, bgFile) ) {
				setBAPIBOMetadata(doc,bgFile);
				
			}else if(getChangeData().sapAPI.equalsIgnoreCase("BAPITXN") && bgFile != null && bgFile.exists() && isTopLevelBOBG(file, bgFile)){
				setBAPITXNBOMetadata(doc,bgFile);
				setBAPITXNAttributeMetadata(doc,getChangeData().attributeMetadataNodeName);
				
			}else if (getChangeData().sapAPI.equalsIgnoreCase("BAPIRS") && isTopLevelBOBG(file, bgFile) && isBAPI_RS_Wrap(file)) {
				setBAPIRSBOMetadata(doc);
				
			}
			
			//modify BO attributes for all SAP APIs
			NodeList nBOAttribute = doc.getElementsByTagNameNS("*", "boAttribute");
			for(int i=0; i<nBOAttribute.getLength(); i++) {
				Element eBOAttribute = (Element)nBOAttribute.item(i);
				createASINodes(eBOAttribute, getChangeData().attributeMetadataNodeName);
			}
			
			writeXml(file, doc);
		}
	}
	
	private void setBAPIBOMetadata(Document doc, IFile bgFile) throws IOException,SAXException{

		DOMParser bgParser = new DOMParser();
		bgParser.parse(bgFile.getLocation().toString());
		Document bgDoc = bgParser.getDocument();
		
		NodeList bgVIElements = bgDoc.getElementsByTagNameNS("*", "VerbsInfo");
		for(int i=0; i<bgVIElements.getLength(); i++) {
			Element bgVIElement = (Element) bgVIElements.item(i);
			if(bgVIElement != null) {
				NodeList verbInfos = bgVIElement.getElementsByTagNameNS("*", "verbInfo");
				NodeList eBODefinitions = doc.getElementsByTagNameNS("*", "boDefinition"); // BO File
				if(verbInfos != null && verbInfos.getLength()>0 && eBODefinitions != null && eBODefinitions.getLength()>0) {
					Element verbInfo = (Element) verbInfos.item(0);
					Element boDefinition = (Element) eBODefinitions.item(0); // BO File
					Element appInfo = (Element) boDefinition.getParentNode(); // BO File
					NodeList boLevelMetadatas = appInfo.getElementsByTagNameNS(getChangeData().namespaceURI + "/metadata", getChangeData().boMetadataNodeName); // BO File
					if(boLevelMetadatas != null && boLevelMetadatas.getLength()>0) {
						Element boLevelMetadata = (Element) boLevelMetadatas.item(0);
						
						//SAP top level BO ASI "Type"
						Element asiType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Type");
						asiType.setPrefix(getChangeData().namespacePrefix);
					    asiType.setTextContent(getChangeData().sapAPI);
						boLevelMetadata.appendChild(asiType);
						
						if(verbInfo.hasAttribute("name") || verbInfo.hasAttribute("info")) {
							Element operation = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Operation");
							operation.setPrefix(getChangeData().namespacePrefix);
							
							//SAP top level BO ASI "MethodName"
							if(verbInfo.hasAttribute("info")) {
								Element methodName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MethodName");
								methodName.setPrefix(getChangeData().namespacePrefix);
								methodName.setTextContent(verbInfo.getAttribute("info"));
								operation.appendChild(methodName);
							}
							//SAP top level BO ASI "Name"
							if(verbInfo.hasAttribute("name")) {
								Element name = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Name");
								name.setPrefix(getChangeData().namespacePrefix);
								name.setTextContent(verbInfo.getAttribute("name"));
								operation.appendChild(name);
							}
							boLevelMetadata.appendChild(operation);
						}
					}
				}
				break;
			}
		}
		
	}
	
	private void setBAPITXNBOMetadata(Document doc, IFile bgFile) throws IOException,SAXException{
		
		DOMParser bgParser = new DOMParser();
		bgParser.parse(bgFile.getLocation().toString());
		Document bgDoc = bgParser.getDocument();
		Element bgVIElement = (Element)bgDoc.getElementsByTagNameNS("*", "VerbsInfo").item(0);
		if(bgVIElement != null) {
			NodeList verbInfos = bgVIElement.getElementsByTagNameNS("*", "verbInfo");
			Element eBODefinition = (Element)doc.getElementsByTagNameNS("*", "boDefinition").item(0); // BO File
			if(verbInfos!=null && eBODefinition!=null){
				Element appInfo = (Element) eBODefinition.getParentNode(); // BO File
				Element boLevelMetadata = (Element)appInfo.getElementsByTagNameNS(getChangeData().namespaceURI + "/metadata", getChangeData().boMetadataNodeName).item(0);
				//SAP top level BO ASI "Type"
				Element asiType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Type");
				asiType.setPrefix(getChangeData().namespacePrefix);
			    asiType.setTextContent(getChangeData().sapAPI);
				boLevelMetadata.appendChild(asiType);
				
				//SAP top level BO ASI "Operation"
				for(int idx=0;idx<verbInfos.getLength();idx++){
					Element verbInfo = (Element)verbInfos.item(idx);
					if(verbInfo.hasAttribute("name") || verbInfo.hasAttribute("info") ){
						Element operation = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Operation");
						operation.setPrefix(getChangeData().namespacePrefix);
						//SAP top level BO ASI "MethodName"
						if(verbInfo.hasAttribute("info")) {
							String infoValues = verbInfo.getAttribute("info");
							if(infoValues!=null && infoValues.indexOf(";")>=0) {
								String[] infoValueSet = infoValues.split(";");
								for(String infoValue : infoValueSet) {
									Element methodName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MethodName");
									methodName.setPrefix(getChangeData().namespacePrefix);
									methodName.setTextContent(infoValue);
									operation.appendChild(methodName);
								}
							}
						}
						//SAP top level BO ASI "Name"
						if(verbInfo.hasAttribute("name")) {
							Element name = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Name");
							name.setPrefix(getChangeData().namespacePrefix);
							name.setTextContent(verbInfo.getAttribute("name"));
							operation.appendChild(name);
						}
						boLevelMetadata.appendChild(operation);
					}
				}
			}
		}
		
	}
	
	private void setBAPITXNAttributeMetadata(Document doc,String metadataNodeName) throws IOException,SAXException,MigrationException{

		String isKey = "";
		String isForeignKey = "";
		NodeList nBOAttribute = doc.getElementsByTagNameNS("*", "boAttribute");
		for(int i=0; i<nBOAttribute.getLength(); i++) {
			Element eBOAttribute = (Element)nBOAttribute.item(i);
			Element eAppInfo = (Element)eBOAttribute.getParentNode();
			Element eTypeMetadata = (Element)eAppInfo.getElementsByTagNameNS(getChangeData().namespaceURI + "/metadata",metadataNodeName).item(0);

			if(eTypeMetadata !=null){
				Element eAttributeInfo = (Element)eBOAttribute.getElementsByTagNameNS("*", "attributeInfo").item(0);
				if(eAttributeInfo.hasAttribute("isKey")) isKey = eAttributeInfo.getAttribute("isKey");
				if(eAttributeInfo.hasAttribute("isForeignKey")) isForeignKey = eAttributeInfo.getAttribute("isForeignKey");
			
				//FieldName
				Element eAnnot = (Element)eAppInfo.getParentNode();
				Element eElement = (Element)eAnnot.getParentNode();
				String strFieldName = this.getFieldName(eElement);
				
				Element fieldName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldName");
				fieldName.setPrefix(getChangeData().namespacePrefix);
				fieldName.setTextContent(strFieldName);
				eTypeMetadata.appendChild(fieldName);
				//FieldType
				Element fieldType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldType");
				fieldType.setPrefix(getChangeData().namespacePrefix);
				fieldType.setTextContent("");
				eTypeMetadata.appendChild(fieldType);
				
				//ForeignKey
				Element foreignKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ForeignKey");
				foreignKey.setPrefix(getChangeData().namespacePrefix);
				if(isKey != null && isForeignKey.equalsIgnoreCase("true"))
					foreignKey.setTextContent("true");
				else
					foreignKey.setTextContent("false");
				eTypeMetadata.appendChild(foreignKey);
				   
				//PrimaryKey
				Element primaryKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "PrimaryKey");
				primaryKey.setPrefix(getChangeData().namespacePrefix);
				if(isKey != null && isKey.equalsIgnoreCase("true"))
				   primaryKey.setTextContent("true");
				else
				   primaryKey.setTextContent("false");
				eTypeMetadata.appendChild(primaryKey);
				
				//PramaterType
				Element parameterType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ParameterType");
				parameterType.setPrefix(getChangeData().namespacePrefix);
				parameterType.setTextContent("INOUT");
				eTypeMetadata.appendChild(parameterType);
				
				//MaxLength
				Element maxLength = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MaxLength");
				maxLength.setPrefix(getChangeData().namespacePrefix);
				maxLength.setTextContent("0");
				eTypeMetadata.appendChild(maxLength);
			}
		}
		
	}

	private String getFieldName(Element eElement) throws SAXException,IOException,MigrationException{
	
		String strFieldName = "";
		String childBOType = eElement.getAttribute("type");
		int idx = childBOType.indexOf(":");
		if(idx>0){
			childBOType = childBOType.substring(idx+1);
		}
		IFile childBOFile = SearchHelper.getMOFileByName(file.getProject(), childBOType+"BG");
		
		DOMParser parser = new DOMParser();
		parser.parse(childBOFile.getLocation().toString());			
		Document childBODoc = parser.getDocument();
		
		Element eVerbInfo = (Element)childBODoc.getElementsByTagNameNS("*", "verbInfo").item(0);
		if(eVerbInfo != null){
			strFieldName = eVerbInfo.getAttribute("info");
		}
		
		return strFieldName;
	}
	
	private void setBAPIRSBOMetadata(Document doc) throws Exception{
		//Find out TopLevel BAPI name and Query BAPI name
		String topLevelBAPIName = null, topLevelBAPIType = null;
		String queryBAPIName = null, queryBAPIType = null;
		NodeList elements = doc.getElementsByTagNameNS("*", "element");
		if(elements!=null) 
		  for(int i=0; i<elements.getLength(); i++) {
			Element element = (Element) elements.item(i);
			if(element.hasAttribute("name") && element.getAttribute("name").equalsIgnoreCase("BAPI_Query")) {
				queryBAPIName = getBAPIName(element); 
				queryBAPIType = getBAPIType(element);
			} else if (element.hasAttribute("name") && element.getAttribute("name").equalsIgnoreCase("BAPI_Result")) {
				topLevelBAPIName = getBAPIName(element); 
				topLevelBAPIType = getBAPIType(element);
			}
		}
		
		//Find Query BAPI file
		IFile queryBAPIFile = null;
		if(queryBAPIName!=null && queryBAPIType!=null) {
			queryBAPIFile = getFileByImportName(file, doc, queryBAPIName, queryBAPIType);
		}

		//Update the targetNameSpace/complexType in QueryBO
	    if(queryBAPIFile!=null && queryBAPIFile.exists()) {
	    	DOMParser qParser = new DOMParser();
			qParser.parse(queryBAPIFile.getLocation().toString());			
			Document qDoc = qParser.getDocument();
			
			NodeList schemas = qDoc.getElementsByTagNameNS("*", "schema");
			if(schemas!=null && schemas.getLength()>0) {
				Element schema = (Element) schemas.item(0);
				if(schema.hasAttribute("targetNamespace")) {
					String targetNamespace = schema.getAttribute("targetNamespace");
					if(targetNamespace.indexOf("/")>=0)
						targetNamespace = targetNamespace.substring(0, targetNamespace.lastIndexOf("/")) + "/querybo";
					schema.setAttribute("targetNamespace", targetNamespace);
				}
			}
			NodeList complexTypes = qDoc.getElementsByTagNameNS("*", "complexType");
			if(complexTypes!=null && complexTypes.getLength()>0) {
				Element complexType = (Element) complexTypes.item(0);
				if(complexType.hasAttribute("name")) {
					complexType.setAttribute("name", "QueryBO");
				}
			}
	    }
		//Rename Query BAPI file to "QueryBO.xsd"
		//Need to rename Query BAPI file after all BO related changes are done.
		
		//Find TopLevel BAPI file
		IFile topLevelBAPIFile = null;
		if(topLevelBAPIName!=null && topLevelBAPIType!=null) {
			topLevelBAPIFile = getFileByImportName(file, doc, topLevelBAPIName, topLevelBAPIType);
		}
		//Create TopLevel BAPI file Metadata & Add "QueryBO" element in TopLevel BAPI file
		if(topLevelBAPIFile!=null && queryBAPIName!=null) {
		   setBAPIRSTopLevelBOMetadata_QueryBOElement(topLevelBAPIFile, topLevelBAPIName, queryBAPIName);
		}
		
	}
	
	private void setBAPIRSTopLevelBOMetadata_QueryBOElement(IFile boFile, String methodNameValue, String queryBAPIName) throws Exception {
		DOMParser parser = new DOMParser();
		parser.parse(boFile.getLocation().toString());
		Document doc = parser.getDocument();
		
		NodeList eBODefinitions = doc.getElementsByTagNameNS("*", "boDefinition");
    	if(eBODefinitions != null && eBODefinitions.getLength()>0) {
			Element boDefinition = (Element) eBODefinitions.item(0); // BO File
			Element appInfo = (Element) boDefinition.getParentNode(); // BO File
			NodeList boLevelMetadatas = appInfo.getElementsByTagNameNS(getChangeData().namespaceURI + "/metadata", getChangeData().boMetadataNodeName); // BO File
			if(boLevelMetadatas != null && boLevelMetadatas.getLength()>0) {
				Element boLevelMetadata = (Element) boLevelMetadatas.item(0);
				
				//SAP top level BO ASI "Type"
				Element asiType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Type");
				asiType.setPrefix(getChangeData().namespacePrefix);
				asiType.setTextContent(getChangeData().sapAPI);
				boLevelMetadata.appendChild(asiType);
						
				Element operation = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Operation");
				operation.setPrefix(getChangeData().namespacePrefix);
				//SAP top level BO ASI "MethodName"
   				Element methodName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MethodName");
				methodName.setPrefix(getChangeData().namespacePrefix);
				methodName.setTextContent(methodNameValue);
				operation.appendChild(methodName);

				//SAP top level BO ASI "Name"
				Element name = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Name");
				name.setPrefix(getChangeData().namespacePrefix);
				name.setTextContent("RetrieveAll");
				operation.appendChild(name);

                boLevelMetadata.appendChild(operation);
			}
		}
    	
    	NodeList sequences = doc.getElementsByTagNameNS("*", "sequence"); 
    	if(sequences!=null && sequences.getLength()>0) {
    		Element sequence = (Element) sequences.item(0);
    		
    		//Write QueryBO Element
    		Element querybo = doc.createElement("element");
    		querybo.setAttribute("name", "QueryBO");
    		querybo.setAttribute("type", "querybo:QueryBO");
    		querybo.setAttribute("nillable", "true");
    		querybo.setAttribute("minOccurs", "0");
    		querybo.setAttribute("maxOccurs", "1");
    		sequence.appendChild(querybo);		
		    //Write Annotation
		    Element annotation = doc.createElement("annotation");
		    annotation.setAttribute("xml:space", "preserve");
		    querybo.appendChild(annotation);
		    //Write appinfo
		    Element appinfo = doc.createElement("appinfo");
		    appinfo.setAttribute("source", getChangeData().namespaceURI + "/metadata");
		    annotation.appendChild(appinfo);
		    //Write sapBAPIPropertyTypeMetadata
		    Element sapBAPIPropertyTypeMetadata = doc.createElementNS(getChangeData().namespaceURI + "/metadata", getChangeData().attributeMetadataNodeName);
		    sapBAPIPropertyTypeMetadata.setPrefix(getChangeData().namespacePrefix);
		    sapBAPIPropertyTypeMetadata.setAttribute("xmlns:sapasi", getChangeData().namespaceURI + "/metadata");
		    appinfo.appendChild(sapBAPIPropertyTypeMetadata);
		    //Write FieldName
		    Element fieldName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldName");
		    fieldName.setPrefix(getChangeData().namespacePrefix);
		    fieldName.setTextContent(queryBAPIName);
		    sapBAPIPropertyTypeMetadata.appendChild(fieldName);
		    //Write FieldType
		    Element fieldType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldType");
		    fieldType.setPrefix(getChangeData().namespacePrefix);
		    fieldType.setTextContent("");
		    sapBAPIPropertyTypeMetadata.appendChild(fieldType);
		    //Write PrimaryKey
		    Element primaryKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "PrimaryKey");
		    primaryKey.setPrefix(getChangeData().namespacePrefix);
            primaryKey.setTextContent("false");
		    sapBAPIPropertyTypeMetadata.appendChild(primaryKey);	
		    //Write ForeignKey
		    Element foreignKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ForeignKey");
		    foreignKey.setPrefix(getChangeData().namespacePrefix);
		    foreignKey.setTextContent("");
		    sapBAPIPropertyTypeMetadata.appendChild(foreignKey);	
		    //Write ParameterType
		    Element parameterType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ParameterType");
		    parameterType.setPrefix(getChangeData().namespacePrefix);
		    parameterType.setTextContent("INOUT");
		    sapBAPIPropertyTypeMetadata.appendChild(parameterType);		     
		    //Write MaxLength
		    Element maxLength = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MaxLength");
		    maxLength.setPrefix(getChangeData().namespacePrefix);
		    maxLength.setTextContent("0");
		    sapBAPIPropertyTypeMetadata.appendChild(maxLength);	
    	}
    	
    	writeXml(boFile, doc);
	}
	
	
	IFile getFileByImportName(IFile file, Document doc, String name, String type) throws MigrationException {

		NodeList schemas = doc.getElementsByTagNameNS("*", "schema");
		String namespace = null;
		if(schemas!=null && schemas.getLength()>0) {
			Element schema = (Element) schemas.item(0);
			String attr = null;
			if(type.indexOf(":")>=0)
				attr = type.substring(0, type.indexOf(":"));
			if(attr!=null && schema.hasAttribute("xmlns:"+attr))
				namespace = schema.getAttribute("xmlns:"+attr);
		}
		NodeList imports = doc.getElementsByTagNameNS("*", "import");
		if(imports!=null && namespace!=null)
			for(int i=0; i<imports.getLength(); i++) {
				Element oneImport = (Element) imports.item(i);
				if(oneImport.hasAttribute("namespace") && oneImport.getAttribute("namespace").equals(namespace)) {
					if(oneImport.hasAttribute("schemaLocation")) {
						String filename = oneImport.getAttribute("schemaLocation");
						ArrayList<IFile> files = SearchHelper.getMigratableBOFiles(file.getProject());
						for(IFile oneFile : files) {
							if(oneFile.getName().equalsIgnoreCase(filename))
									return oneFile;
						}
					}
				}
			}
		return null;
	}
	
	
	String getBAPIType(Element element) {
		if(element.hasAttribute("type"))
			return element.getAttribute("type");
		return null;
	}
	
	String getBAPIName(Element element) {
		NodeList appSpecificInfos = element.getElementsByTagNameNS("*", "appSpecificInfo");
		if(appSpecificInfos!=null && appSpecificInfos.getLength()>0) {
			Element appSpecificInfo = (Element) appSpecificInfos.item(0);
			String value = appSpecificInfo.getTextContent();
			if(value!=null && value.length()>0) {
				String[] values = value.split(getChangeData().keyPairsSplitRegEx);
				for(String oneValue: values) {
					if(oneValue.toLowerCase().startsWith("bapi=")) {
						return oneValue.substring(5);
					}
				}
			}
		}
		return null;
	}
	
	boolean isBAPI_RS_Wrap(IFile file) throws SAXException, IOException {
		boolean ret = false;
		int count = 0;
		
		DOMParser parser = new DOMParser();
		parser.parse(file.getLocation().toString());
		Document doc = parser.getDocument();
		
		NodeList elements = doc.getElementsByTagNameNS("*", "element");
		if(elements!=null)
		   for(int i=0; i<elements.getLength(); i++) {
			  Element element = (Element) elements.item(i);
			  if(element.hasAttribute("name"))
				  if(element.getAttribute("name").equalsIgnoreCase("BAPI_Query"))
					  count++;
				  else if(element.getAttribute("name").equalsIgnoreCase("BAPI_Result"))
					  count++;
		   }
		if(count>=2) ret = true;
		return ret;
	}
	
	protected void createASINodes(Element eASIDefinition, String metadataNodeName) throws Exception {
		Document doc = eASIDefinition.getOwnerDocument();
		
		Element eAppInfo = (Element)eASIDefinition.getParentNode();
		Element eAnnotation = (Element)eAppInfo.getParentNode();
		Element eElement = (Element)eAnnotation.getParentNode();
		
		//CR93069 - Move it out of the if condition.
		// Add the <SimpleType> part
		String maxLengthValue = "";
		NodeList eSimpleTypes = eElement.getElementsByTagNameNS("*", "simpleType");
		if(eSimpleTypes != null && eSimpleTypes.getLength()>0) {
			NodeList eRestrictions = ((Element) eSimpleTypes.item(0)).getElementsByTagNameNS("*", "restriction");
			if(eRestrictions != null && eRestrictions.getLength()>0) {
				NodeList eMaxLength = ((Element) eRestrictions.item(0)).getElementsByTagNameNS("*", "maxLength");
				if(eMaxLength != null && eMaxLength.getLength()>0) {
					String mlvalue = ((Element) eMaxLength.item(0)).getAttribute("value");
					if(mlvalue != null && mlvalue.length()>0) {
						maxLengthValue = mlvalue;
						
						Element simpleTypeE = doc.createElement("simpleType");
						Element restrictionE = doc.createElement("restriction");
						restrictionE.setAttribute("base", "string");
						Element maxLengthE = doc.createElement("maxLength");
						maxLengthE.setAttribute("value", mlvalue);
						restrictionE.appendChild(maxLengthE);
						simpleTypeE.appendChild(restrictionE);
						eElement.appendChild(simpleTypeE);
						
						//eElement needs to remove the "type" attribute
						if(eElement.hasAttribute("type"))
						   eElement.removeAttribute("type");
					}
				}
			}
		}
		
		NodeList eAppSpecificInfos = eASIDefinition.getElementsByTagNameNS("*", "appSpecificInfo");
		Element eAppSpecificInfo = null;
		if(eAppSpecificInfos.getLength()>0) eAppSpecificInfo = (Element) eAppSpecificInfos.item(0);
		String sapAPI = ((ConvertWBIASItoJCAASIForSAP)getChangeData()).sapAPI;
		
		NodeList eAttributeInfos = eASIDefinition.getElementsByTagNameNS("*", "attributeInfo");
		Element eAttributeInfo = null;
		if(eAttributeInfos.getLength()>0) eAttributeInfo = (Element) eAttributeInfos.item(0);
		String isKey = "";
		if(eAttributeInfo.hasAttribute("isKey")) isKey = eAttributeInfo.getAttribute("isKey");
		String isForeignKey = "";
		if(eAttributeInfo.hasAttribute("isForeignKey")) isForeignKey = eAttributeInfo.getAttribute("isForeignKey");
		
		if(eAppSpecificInfo!=null) {		
			NodeList eTypeMetadatas = null;
			if(sapAPI.equalsIgnoreCase("BAPI") || sapAPI.equalsIgnoreCase("BAPITXN") || sapAPI.equalsIgnoreCase("BAPIRS")) 
			{
				AttributeMap<String> asiConfigElements = getSapBAPIASIConfigElements(eAppSpecificInfo);
				
				eTypeMetadatas = eAppInfo.getElementsByTagNameNS(getChangeData().namespaceURI + "/metadata", getChangeData().attributeMetadataNodeName);
			    if(eTypeMetadatas!=null && eTypeMetadatas.getLength()>0 && asiConfigElements.keySet().size()>0)
				{
					String left = (String) asiConfigElements.keySet().toArray()[0];
					String right = (String) asiConfigElements.get(left);
					String type = "", value = "";
					if(left!=null && left.startsWith("I") && right!=null && right.startsWith("E"))
					{ type = "INOUT"; value = left.substring(1); }
					else if(left!=null && left.startsWith("I"))
				    { type = "IN"; value = left.substring(1); }
					else if(right!=null && right.startsWith("E"))
					{ type = "OUT"; value = right.substring(1); }
					else throw new Exception("BAPI ASI for SAP Adapter" + left + ":" + right + " is not recognized.");
					
					//CR93069 - Move it out of the if condition.
					// Add the <SimpleType> part
//					String maxLengthValue = "";
//					if(eSimpleTypes != null && eSimpleTypes.getLength()>0) {
//						NodeList eRestrictions = ((Element) eSimpleTypes.item(0)).getElementsByTagNameNS("*", "restriction");
//						if(eRestrictions != null && eRestrictions.getLength()>0) {
//							NodeList eMaxLength = ((Element) eRestrictions.item(0)).getElementsByTagNameNS("*", "maxLength");
//							if(eMaxLength != null && eMaxLength.getLength()>0) {
//								String mlvalue = ((Element) eMaxLength.item(0)).getAttribute("value");
//								if(mlvalue != null && mlvalue.length()>0) {
//									maxLengthValue = mlvalue;
//									
//									Element simpleTypeE = doc.createElement("simpleType");
//									Element restrictionE = doc.createElement("restriction");
//									restrictionE.setAttribute("base", "string");
//									Element maxLengthE = doc.createElement("maxLength");
//									maxLengthE.setAttribute("value", mlvalue);
//									restrictionE.appendChild(maxLengthE);
//									simpleTypeE.appendChild(restrictionE);
//									eElement.appendChild(simpleTypeE);
//									
//									//eElement needs to remove the "type" attribute
//									if(eElement.hasAttribute("type"))
//									   eElement.removeAttribute("type");
//								}
//							}
//						}
//					}
					
					Element fieldName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldName");
					fieldName.setPrefix(getChangeData().namespacePrefix);
					fieldName.setTextContent(value);
					eTypeMetadatas.item(0).appendChild(fieldName);
					
					Element fieldType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldType");
					fieldType.setPrefix(getChangeData().namespacePrefix);
					fieldType.setTextContent("");
					eTypeMetadatas.item(0).appendChild(fieldType);
					
					//Add Foreign key - Different SAP APIs have different foreign key settings
					if(sapAPI.equalsIgnoreCase("BAPI") || sapAPI.equalsIgnoreCase("BAPITXN")) {
					   Element foreignKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ForeignKey");
					   foreignKey.setPrefix(getChangeData().namespacePrefix);
					   if(isKey != null && isForeignKey.equalsIgnoreCase("true"))
						   foreignKey.setTextContent("true");
					   else
						   foreignKey.setTextContent("false");
					   eTypeMetadatas.item(0).appendChild(foreignKey);
					   
					} else if(sapAPI.equalsIgnoreCase("BAPIRS")) {
						if(BAPI_RS_FK_Value!=null) {
							   Element foreignKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ForeignKey");
							   foreignKey.setPrefix(getChangeData().namespacePrefix);
							   foreignKey.setTextContent(BAPI_RS_FK_Value);
							   eTypeMetadatas.item(0).appendChild(foreignKey);
							   BAPI_RS_FK_Value = null;
						}
					}
					
					Element primaryKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "PrimaryKey");
					primaryKey.setPrefix(getChangeData().namespacePrefix);
					if(isKey != null && isKey.equalsIgnoreCase("true"))
					   primaryKey.setTextContent("true");
					else
					   primaryKey.setTextContent("false");
					eTypeMetadatas.item(0).appendChild(primaryKey);
					
					Element parameterType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ParameterType");
					parameterType.setPrefix(getChangeData().namespacePrefix);
					parameterType.setTextContent(type);
					eTypeMetadatas.item(0).appendChild(parameterType);
					
					Element maxLength = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MaxLength");
					maxLength.setPrefix(getChangeData().namespacePrefix);
					if(maxLengthValue==null || maxLengthValue == ""){
					//Set the MaxLength to '0' if there is no value specified.
						maxLengthValue = "0";
					}
					maxLength.setTextContent(maxLengthValue);
					eTypeMetadatas.item(0).appendChild(maxLength);
				}
			} else if(sapAPI.equalsIgnoreCase("ALE")) {
				String asiContent = eAppSpecificInfo.getTextContent();
				String[] values = null;
				if(asiContent!=null && asiContent.indexOf(getChangeData().boASIKeyPairsSplitRegEx)>=0)
					values = asiContent.split(getChangeData().boASIKeyPairsSplitRegEx);
				boolean isDummyKey = false;
				if(values!=null)
					for(String value : values) {
						if(value.equalsIgnoreCase("DummyKey"))
						{ isDummyKey = true; break; }
					}

				if(isDummyKey) {
					//This is ALE DummyKey ASI, which need specially process
					eElement.setAttribute("name", "DummyKey");
					//eElement.setAttribute("type", "string");
					eElement.setAttribute("nillable", "true");
					eElement.setAttribute("minOccurs", "1");
					eElement.setAttribute("maxOccurs", "1");
					
					eTypeMetadatas = eElement.getElementsByTagNameNS(getChangeData().namespaceURI + "/metadata", getChangeData().attributeMetadataNodeName);
					if(eTypeMetadatas!=null && eTypeMetadatas.getLength()>0) {
						
						Element fieldName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldName");
						fieldName.setPrefix(getChangeData().namespacePrefix);
						fieldName.setTextContent("DummyKey");
						eTypeMetadatas.item(0).appendChild(fieldName);
						
						Element segmentHierarchy = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "SegmentHierarchy");
						segmentHierarchy.setPrefix(getChangeData().namespacePrefix);
						segmentHierarchy.setTextContent("0");
						eTypeMetadatas.item(0).appendChild(segmentHierarchy);
						
						Element offSet = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "OffSet");
						offSet.setPrefix(getChangeData().namespacePrefix);
						offSet.setTextContent("0");
						eTypeMetadatas.item(0).appendChild(offSet);
						
						Element primaryKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "PrimaryKey");
						primaryKey.setPrefix(getChangeData().namespacePrefix);
						if(isKey != null && isKey.equalsIgnoreCase("true"))
						   primaryKey.setTextContent("true");
						else
						   primaryKey.setTextContent("false");
						eTypeMetadatas.item(0).appendChild(primaryKey);
						
						Element foreignBOKeyRef = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ForeignBOKeyRef");
						foreignBOKeyRef.setPrefix(getChangeData().namespacePrefix);
						foreignBOKeyRef.setTextContent("");
						for(String value : values) {
							if(value.startsWith("FK=") || value.startsWith("FK-")) {
								foreignBOKeyRef.setTextContent(value.substring(3).replaceAll(":", "/"));
								break; 
							}
						}
						eTypeMetadatas.item(0).appendChild(foreignBOKeyRef);
						
						Element maxLength = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MaxLength");
						maxLength.setPrefix(getChangeData().namespacePrefix);
						maxLength.setTextContent("0");
						eTypeMetadatas.item(0).appendChild(maxLength);
					}
					
					
				} else {
				  eTypeMetadatas = eAppInfo.getElementsByTagNameNS(getChangeData().namespaceURI + "/metadata", getChangeData().attributeMetadataNodeName);
			      if(eTypeMetadatas!=null && eTypeMetadatas.getLength()>0 && asiContent!=null && asiContent.length()>0) {
			    	String offsetValue = "", fieldNameValue = "";
			    	if(asiContent.indexOf("+")>=0) {
			    		offsetValue = asiContent.substring(0, asiContent.indexOf("+"));
			    		fieldNameValue = asiContent.substring(asiContent.indexOf("+")+1);
			    	} else
			    		fieldNameValue = asiContent;
			    	if(offsetValue==null || offsetValue.length()==0)
			    		offsetValue = "0";
			    	
			    	//CR93069 - Move it out of the if condition.
			    	// Add the <SimpleType> part
//					String maxLengthValue = "";
//					if(eSimpleTypes != null && eSimpleTypes.getLength()>0) {
//						NodeList eRestrictions = ((Element) eSimpleTypes.item(0)).getElementsByTagNameNS("*", "restriction");
//						if(eRestrictions != null && eRestrictions.getLength()>0) {
//							NodeList eMaxLength = ((Element) eRestrictions.item(0)).getElementsByTagNameNS("*", "maxLength");
//							if(eMaxLength != null && eMaxLength.getLength()>0) {
//								String mlvalue = ((Element) eMaxLength.item(0)).getAttribute("value");
//								if(mlvalue != null && mlvalue.length()>0) {
//									maxLengthValue = mlvalue;
//									
//									Element simpleTypeE = doc.createElement("simpleType");
//									Element restrictionE = doc.createElement("restriction");
//									restrictionE.setAttribute("base", "string");
//									Element maxLengthE = doc.createElement("maxLength");
//									maxLengthE.setAttribute("value", mlvalue);
//									restrictionE.appendChild(maxLengthE);
//									simpleTypeE.appendChild(restrictionE);
//									eElement.appendChild(simpleTypeE);
//									
//									//eElement needs to remove the "type" attribute
//									if(eElement.hasAttribute("type"))
//									   eElement.removeAttribute("type");
//								}
//							}
//						}
//					}
					
					Element fieldName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldName");
					fieldName.setPrefix(getChangeData().namespacePrefix);
					fieldName.setTextContent(fieldNameValue);
					eTypeMetadatas.item(0).appendChild(fieldName);
					
					Element segmentHierarchy = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "SegmentHierarchy");
					segmentHierarchy.setPrefix(getChangeData().namespacePrefix);
					segmentHierarchy.setTextContent("0");
					eTypeMetadatas.item(0).appendChild(segmentHierarchy);
					
					Element offSet = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "OffSet");
					offSet.setPrefix(getChangeData().namespacePrefix);
					offSet.setTextContent(offsetValue);
					eTypeMetadatas.item(0).appendChild(offSet);
					
					Element primaryKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "PrimaryKey");
					primaryKey.setPrefix(getChangeData().namespacePrefix);
					if(isKey != null && isKey.equalsIgnoreCase("true"))
					   primaryKey.setTextContent("true");
					else
					   primaryKey.setTextContent("false");
					eTypeMetadatas.item(0).appendChild(primaryKey);
					
					Element foreignBOKeyRef = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ForeignBOKeyRef");
					foreignBOKeyRef.setPrefix(getChangeData().namespacePrefix);
					foreignBOKeyRef.setTextContent("");
					eTypeMetadatas.item(0).appendChild(foreignBOKeyRef);
					
					Element maxLength = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MaxLength");
					maxLength.setPrefix(getChangeData().namespacePrefix);
					maxLength.setTextContent(maxLengthValue);
					eTypeMetadatas.item(0).appendChild(maxLength);
			      }
				}
		    } 
		}
		
	}
	
	protected AttributeMap<String> getSapBAPIASIConfigElements(Element eAppSpecificInfo) {
		AttributeMap<String> asiConfigElements = new AttributeMap<String>(AttributeMap.Type.IGNORE_CASE);
		
		String sAsiConfigElements = eAppSpecificInfo.getTextContent();
		String [] asiConfigElementsArray = sAsiConfigElements.split(getChangeData().keyPairsSplitRegEx);
		for(String asiConfigElement : asiConfigElementsArray) {
			
			if(getChangeData().sapAPI.equalsIgnoreCase("BAPIRS") && asiConfigElement.toUpperCase().startsWith("FK=")) {
				String value = asiConfigElement.substring(3).replaceAll(":", "/");
				if(value.indexOf("/")>=0)
				    BAPI_RS_FK_Value = "QueryBO" + value.substring(value.indexOf("/"));
				else
					BAPI_RS_FK_Value = "QueryBO";
				
			} else {
			   String [] asiConfigElementPair = asiConfigElement.split(":");
			   String key = asiConfigElementPair[0];
			   String value = asiConfigElementPair.length>1 ? asiConfigElementPair[1] : "";
			   asiConfigElements.put(key, value);
			}
			/*
			if(getChangeData().valueSplitRegEx!=null) {
				String [] splitValues = value.split(getChangeData().valueSplitRegEx);
				if(splitValues.length>1) {
					for(int i=0; i<splitValues.length; i++) {
						asiConfigElements.put(key + "[" + i + "]", splitValues[i]);
					}
				}
				else {
					asiConfigElements.put(key, value);
				}
			}
			else {			
				asiConfigElements.put(key, value);
			}
			*/
			
		}
		
		return asiConfigElements;
	}
	

	protected boolean isTopLevelBOBG(IFile bofile, IFile bgfile) throws CoreException, IOException, SAXException, MigrationException {
		boolean ret = false;
		if(topLevelBOs == null)
           topLevelBOs = SearchHelper.getTopLevelBOs(bofile.getProject());
		
		for(IFile file : topLevelBOs) {
			if(bofile.getLocation().toString().equalsIgnoreCase(file.getLocation().toString()))
			{ ret = true; break; }
		}
		
		if(ret == true) {
		   DOMParser bgParser = new DOMParser();
		   bgParser.parse(bgfile.getLocation().toString());
		   Document bgDoc = bgParser.getDocument();
		   NodeList elements = bgDoc.getElementsByTagNameNS("*", "element");
		   
		   DOMParser parser = new DOMParser();
		   parser.parse(file.getLocation().toString());			
		   Document doc = parser.getDocument();
		   NodeList complexTypes = doc.getElementsByTagNameNS("*", "complexType");
		   
		   ret = false;
		   if(complexTypes != null && complexTypes.getLength()>0) {
			   Element complexType = (Element) complexTypes.item(0);
			   
			   if(elements != null && complexType.hasAttribute("name"))
			      for(int i=0; i<elements.getLength(); i++) {
			    	  Element element = (Element) elements.item(i);
			    	  if(element.hasAttribute("name")
			    		  && element.getAttribute("name").equals(complexType.getAttribute("name")))
			    	  { ret = true; break; }
			      }
		   }
		}
		return ret;
	}
}
