/* IBM Confidential
 * OCO Source Materials
 * 5724-N40, 5724-N41, 5724-N42, 5724-N43, 5724-L77, 5724-L78, 5724-L79, 5724-L80, 5724-L81, 5724-L82
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.migration.sap.wbi_to_v610;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.j2ca.migration.ArtifactChangeArguments;
import com.ibm.j2ca.migration.IChangeArguments;
import com.ibm.j2ca.migration.internal.changes.CreateFileChange;
import com.ibm.j2ca.migration.sap.v602_to_v610.MigrationMessages;
import com.ibm.j2ca.migration.util.SearchHelper;
import com.ibm.j2ca.migration.util.Util;

public class CreateBOWrapperForSAPChange extends CreateFileChange {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$
	
	private IFile file;
	private IProject project;
	private String boName = null;
	private ArrayList<IFile> topLevelBOs = null;
	
	public CreateBOWrapperForSAPChange(IProject project, CreateBOWrapperForSAP changeData) {
		super(project, changeData);
		this.project = project;
	}
	
	public String getChangeDetails() {
		return MigrationMessages.CreateBOWrapperForSAPChange_Description;
	}
	
	public CreateBOWrapperForSAP getChangeData() { return (CreateBOWrapperForSAP)super.getChangeData(); }
	
	public void perform(IProgressMonitor pm) throws Exception {
		if(topLevelBOs == null)
			topLevelBOs = SearchHelper.getSupportedBOs(project);
		
		if(topLevelBOs.size()>0) {
			file = topLevelBOs.get(0);
		    String targetFileLocation = getTargetLocation();
			//CR92961 - Skip it if the target file already exists.
			if(new File(targetFileLocation).exists()) { return; }

		    copy(getChangeData().sourceUrl, targetFileLocation, true);
            getProject().refreshLocal(IResource.DEPTH_INFINITE, pm);
            for(IProject sharedProject : SearchHelper.getSharedProjects(project)) {
            	sharedProject.refreshLocal(IResource.DEPTH_INFINITE, pm);
            }
        
            if(beforeCreate(pm)) {
	           IFile targetFile = SearchHelper.getFile(getProject(), targetFileLocation);
	           afterCreate(targetFile, pm);
            }
		}
	}
	
	public IChangeArguments getChangeArguments() {
		IContainer targetContainer = getTargetContainer();
		String fileLocation = getTargetLocation();
		String fileName = fileLocation.substring(fileLocation.lastIndexOf('/') + 1);
		
		ArtifactChangeArguments argument = new ArtifactChangeArguments(targetContainer.getFile(new Path(fileName)));
		return argument;
	}
	
	protected String getTargetLocation() {
		
		if(topLevelBOs == null) {
			try { 
				topLevelBOs = SearchHelper.getTopLevelBOs(project);
				if(topLevelBOs != null)
					   return topLevelBOs.get(0).getParent().getLocation().toString() + "/" + getWrapperBOName() + ".xsd";

			} catch(Exception e) {
			}
		} else {
			return topLevelBOs.get(0).getParent().getLocation().toString() + "/" + getWrapperBOName() + ".xsd";
		}
		
		return project.getLocation().toString() + "/" + getWrapperBOName() + ".xsd";
	
		/*
		String fileLocation = getChangeData().sourceUrl.getFile();

		String fileName = null;
		try {
			fileName = getWrapperBOName() + ".xsd";
		}
		catch(Exception e) {
			fileName = fileLocation.substring(fileLocation.lastIndexOf('/'));
		}
		
		IContainer targetContainer = getTargetContainer();
		return targetContainer.getLocation().toString() + "/" + fileName;
		*/
	}
	
	protected String getBOName() throws IOException, SAXException {
		if(boName==null) {
			boName = Util.getBusinessObjectName(file).replaceAll("BG$", "");
		}
		return boName;
	}
	
	protected String getWrapperBOName() {
		//return getBOName() + "Wrapper";
		if(getChangeData().sapAPI.startsWith("BAPI"))
			   return "BAPIWrapper";
		else if (getChangeData().sapAPI.equalsIgnoreCase("ALE"))
			   return "ALEWrapper";
		else return "Wrapper";
	}
	
	public void afterCreate(IFile targetFile, IProgressMonitor pm) throws Exception {
        
		//String baseNamespace = ((CreateBOWrapperForSAP)this.getChangeData()).baseNamespace; 
        //if(baseNamespace==null){
        //	baseNamespace = Util.getBaseNamespace(getProject(), targetFile.getLocation().toString());        
        //}
        //String boNamespace = Util.getTargetNamespace(file);
		
		DOMParser parser = new DOMParser();	
		parser.parse(targetFile.getLocation().toString());				
		Document doc = parser.getDocument();
		boolean isTypeSet = false;
		boolean isFirstItem = true;
		String methodName_fieldName = "";
		
        for(IFile file : topLevelBOs) {  
           if(!file.getLocation().toString().endsWith("WICSFault.xsd")) {
		
		     Element eSchema = (Element)doc.getElementsByTagNameNS("*", "schema").item(0);
		     String boName = Util.getBusinessObjectName(file);
		     String targetNamespace = Util.getTargetNamespace(file);
		     if(eSchema!=null && boName!=null && targetNamespace!=null) {
		        eSchema.setAttribute("xmlns:" + boName.toLowerCase(), targetNamespace);
		     }
		
		     Element eImport = (Element)doc.createElement("import");
		     if(eImport!=null) {
			    eImport.setAttribute("namespace", targetNamespace);
			    eImport.setAttribute("schemaLocation", file.getName());
		     }
		     if(eSchema.getChildNodes().getLength()>0) 
		         eSchema.insertBefore(eImport, eSchema.getFirstChild());
		     else
		    	 eSchema.appendChild(eImport);
		
		     Element eBOMetadata = (Element)eSchema.getElementsByTagNameNS("*", getChangeData().boMetadataNodeName).item(0);
		     
		     if(eBOMetadata!=null && getChangeData().sapAPI.startsWith("BAPI")) {
			    DOMParser fileParser = new DOMParser();	
			    fileParser.parse(file.getLocation().toString());				
			    Document fileDoc = fileParser.getDocument();
			    
			    NodeList complexTypes = fileDoc.getElementsByTagNameNS("*", "complexType");
			    if(complexTypes!=null && complexTypes.getLength()>0){
			    	Element complexType = (Element) complexTypes.item(0);
			    	NodeList sourceBOMetadatas = complexType.getElementsByTagNameNS("*", getChangeData().boMetadataNodeName);
			    	if(sourceBOMetadatas!=null && sourceBOMetadatas.getLength()>0) {
			    		Element sourceBOMetadata = (Element) sourceBOMetadatas.item(0);
			    		// Set SAP Metadata "Type"
			    		if(isTypeSet == false) {
			    			NodeList types = sourceBOMetadata.getElementsByTagNameNS("*", "Type");
			    			if(types!=null && types.getLength()>0) {
			    				Element wrapType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Type");
			    				wrapType.setPrefix(getChangeData().namespacePrefix);
			    				wrapType.setTextContent(((Element) types.item(0)).getTextContent());
			    				eBOMetadata.appendChild(wrapType);
			    				isTypeSet = true;
			    			}
			    		}
			    		// Set SAP Metadata "Operation"
			    		NodeList operations = sourceBOMetadata.getElementsByTagNameNS("*", "Operation");
			    		if(operations!=null && operations.getLength()>0) {
			    			Element operation = (Element) operations.item(0);
			    			
			    			//Write the "Operation"
			    			Element wrapOperation = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Operation");
			    			wrapOperation.setPrefix(getChangeData().namespacePrefix);
			    			eBOMetadata.appendChild(wrapOperation);
			    			
			    			//Write the "MethodName"
			    			NodeList methodNames = operation.getElementsByTagNameNS("*", "MethodName");
			    			if(methodNames!=null && methodNames.getLength()>0) {
			    				Element wrapMethodName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MethodName");
			    				wrapMethodName.setPrefix(getChangeData().namespacePrefix);
			    				wrapMethodName.setTextContent(((Element) methodNames.item(0)).getTextContent());
			    				methodName_fieldName = ((Element) methodNames.item(0)).getTextContent();
			    				wrapOperation.appendChild(wrapMethodName);
			    			}
			    			
			    			//Write the "Name"
			    			NodeList names = operation.getElementsByTagNameNS("*", "Name");
			    			if(names!=null && names.getLength()>0) {
			    				Element wrapName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Name");
			    				wrapName.setPrefix(getChangeData().namespacePrefix);
			    				wrapName.setTextContent(((Element) names.item(0)).getTextContent());
			    				wrapOperation.appendChild(wrapName);
			    			}
			    		}
			    	}
			    }
		     } else if(eBOMetadata!=null && getChangeData().sapAPI.equalsIgnoreCase("ALE")) {
		    	
		    	Element wrapType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Type");
 				wrapType.setPrefix(getChangeData().namespacePrefix);
 				wrapType.setTextContent("IDOC");
 				eBOMetadata.appendChild(wrapType);
 				
 				String bgFileLocation = file.getLocation().toString().substring(0, file.getLocation().toString().lastIndexOf(".")) + "BG.xsd";
 				IFile bgFile = SearchHelper.getFile(file.getProject(), bgFileLocation);

 				if(bgFile != null && bgFile.exists()) {
				   DOMParser bgParser = new DOMParser();
				   bgParser.parse(bgFile.getLocation().toString());
				   Document bgDoc = bgParser.getDocument();
				   
				   NodeList verbInfos = bgDoc.getElementsByTagNameNS("*", "verbInfo");
				   if(verbInfos!=null)
					   for(int i=0; i<verbInfos.getLength(); i++) {
						   Element verbInfo = (Element) verbInfos.item(i);
						   if(verbInfo.hasAttribute("name") && verbInfo.hasAttribute("info")) {
							   String verb = verbInfo.getAttribute("name");
							   String info = verbInfo.getAttribute("info");
							   if(verb.equalsIgnoreCase("Create") || verb.equalsIgnoreCase("Update") || verb.equalsIgnoreCase("Delete") || verb.equalsIgnoreCase("Retrieve")) {
								   Element wrapOperation = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Operation");
					    		   wrapOperation.setPrefix(getChangeData().namespacePrefix);
					    		   eBOMetadata.appendChild(wrapOperation);
					    		   
					    		   Element wrapName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "Name");
					    		   wrapName.setPrefix(getChangeData().namespacePrefix);
					    		   wrapName.setTextContent(verb);
					    		   wrapOperation.appendChild(wrapName);
					    		   
					    		   Element sapALEOperationMetadata = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "sapALEOperationMetadata");
					    		   sapALEOperationMetadata.setPrefix(getChangeData().namespacePrefix);
					    		   wrapOperation.appendChild(sapALEOperationMetadata);
					    		   
					    		   String[] infoValues = info.replaceAll(",", ";").split(";");
					    		   if(infoValues!=null)
					    		     for(String infoValue : infoValues) {
					    		    	 if(infoValue.startsWith("MsgType=")) {
					    		    		   Element msgType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MsgType");
					    		    		   msgType.setPrefix(getChangeData().namespacePrefix);
					    		    		   msgType.setTextContent(infoValue.substring(8));
								    		   sapALEOperationMetadata.appendChild(msgType);
					    		    	 } else if(infoValue.startsWith("MsgCode=")) {
					    		    		   Element msgCode = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MsgCode");
					    		    		   msgCode.setPrefix(getChangeData().namespacePrefix);
					    		    		   msgCode.setTextContent(infoValue.substring(8));
								    		   sapALEOperationMetadata.appendChild(msgCode);
					    		    	 } else if(infoValue.startsWith("MsgFunction=")) {
					    		    		   Element msgFunction = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MsgFunction");
					    		    		   msgFunction.setPrefix(getChangeData().namespacePrefix);
					    		    		   msgFunction.setTextContent(infoValue.substring(12));
								    		   sapALEOperationMetadata.appendChild(msgFunction);
					    		    	 } 
					    		     }
							   }
						   }
					   }
 				}
		     }
		     
		     Element eSequence = (Element)eSchema.getElementsByTagNameNS("*", "sequence").item(0);
		     //Write element
		     Element element = doc.createElement("element");
		     element.setAttribute("name", boName);
		     element.setAttribute("type", boName.toLowerCase()+":"+boName);
		     element.setAttribute("nillable", "true");
		     element.setAttribute("minOccurs", "0");
		     element.setAttribute("maxOccurs", "1");
		     eSequence.appendChild(element);
		     //Write Annotation
		     Element annotation = doc.createElement("annotation");
		     annotation.setAttribute("xml:space", "preserve");
		     element.appendChild(annotation);
		     //Write appinfo
		     Element appinfo = doc.createElement("appinfo");
		     appinfo.setAttribute("source", getChangeData().namespaceURI + "/metadata");
		     annotation.appendChild(appinfo);
		     
		     if(getChangeData().sapAPI.startsWith("BAPI")) {
		        //Write sapBAPIPropertyTypeMetadata
		        Element sapBAPIPropertyTypeMetadata = doc.createElementNS(getChangeData().namespaceURI + "/metadata", getChangeData().attributeMetadataNodeName);
		        sapBAPIPropertyTypeMetadata.setPrefix(getChangeData().namespacePrefix);
		        sapBAPIPropertyTypeMetadata.setAttribute("xmlns:sapasi", getChangeData().namespaceURI + "/metadata");
		        appinfo.appendChild(sapBAPIPropertyTypeMetadata);
		        //Write FieldName
		        Element fieldName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldName");
		        fieldName.setPrefix(getChangeData().namespacePrefix);
		        fieldName.setTextContent(methodName_fieldName);
		        sapBAPIPropertyTypeMetadata.appendChild(fieldName);
		        //Write FieldType
		        Element fieldType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldType");
		        fieldType.setPrefix(getChangeData().namespacePrefix);
		        fieldType.setTextContent("");
		        sapBAPIPropertyTypeMetadata.appendChild(fieldType);
		        //Write PrimaryKey
		        Element primaryKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "PrimaryKey");
		        primaryKey.setPrefix(getChangeData().namespacePrefix);
		        if(isFirstItem)
		           { primaryKey.setTextContent("true"); isFirstItem = false; }
		        else
		           { primaryKey.setTextContent("false"); } 
		        sapBAPIPropertyTypeMetadata.appendChild(primaryKey);		     
		        //Write ParameterType
		        Element parameterType = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ParameterType");
		        parameterType.setPrefix(getChangeData().namespacePrefix);
		        parameterType.setTextContent("INOUT");
		        sapBAPIPropertyTypeMetadata.appendChild(parameterType);		     
		        //Write MaxLength
		        Element maxLength = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MaxLength");
		        maxLength.setPrefix(getChangeData().namespacePrefix);
		        maxLength.setTextContent("0");
		        sapBAPIPropertyTypeMetadata.appendChild(maxLength);
		        
		     } else if(getChangeData().sapAPI.equalsIgnoreCase("ALE")) {
		    	//Write sapBAPIPropertyTypeMetadata
			    Element sapALEPropertyTypeMetadata = doc.createElementNS(getChangeData().namespaceURI + "/metadata", getChangeData().attributeMetadataNodeName);
			    sapALEPropertyTypeMetadata.setPrefix(getChangeData().namespacePrefix);
			    sapALEPropertyTypeMetadata.setAttribute("xmlns:sapasi", getChangeData().namespaceURI + "/metadata");
			    appinfo.appendChild(sapALEPropertyTypeMetadata);
			    
			    Element fieldName = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "FieldName");
			    fieldName.setPrefix(getChangeData().namespacePrefix);
			    fieldName.setTextContent(boName);
			    sapALEPropertyTypeMetadata.appendChild(fieldName);
			        
			    Element segmentHierarchy = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "SegmentHierarchy");
				segmentHierarchy.setPrefix(getChangeData().namespacePrefix);
				segmentHierarchy.setTextContent("0");
				sapALEPropertyTypeMetadata.appendChild(segmentHierarchy);
					
				Element offSet = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "OffSet");
				offSet.setPrefix(getChangeData().namespacePrefix);
				offSet.setTextContent("0");
				sapALEPropertyTypeMetadata.appendChild(offSet);
					
				Element primaryKey = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "PrimaryKey");
				primaryKey.setPrefix(getChangeData().namespacePrefix);
				primaryKey.setTextContent("true");
				sapALEPropertyTypeMetadata.appendChild(primaryKey);
					
				Element foreignBOKeyRef = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "ForeignBOKeyRef");
				foreignBOKeyRef.setPrefix(getChangeData().namespacePrefix);
				foreignBOKeyRef.setTextContent("");
				sapALEPropertyTypeMetadata.appendChild(foreignBOKeyRef);
					
				Element maxLength = doc.createElementNS(getChangeData().namespaceURI + "/metadata", "MaxLength");
				maxLength.setPrefix(getChangeData().namespacePrefix);
				maxLength.setTextContent("0");
				sapALEPropertyTypeMetadata.appendChild(maxLength);
		     }
           }
        }
		writeXml(targetFile, doc);
	}

}
