/* IBM Confidential
 * OCO Source Materials
 * 5724-N40, 5724-N41, 5724-N42, 5724-N43, 5724-L77, 5724-L78, 5724-L79, 5724-L80, 5724-L81, 5724-L82
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */

package com.ibm.j2ca.migration.sap.wbi_to_v610;

import java.io.IOException;
import java.util.Hashtable;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.j2ca.migration.changedata.wbi.ImportWBIProperties;
import com.ibm.j2ca.migration.internal.changes.wbi.ImportWBIPropertiesChange;
import com.ibm.j2ca.migration.util.DecryptPropertyValue;

public class ImportWBIPropertiesForSAPChange  extends ImportWBIPropertiesChange {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$
	
	
	public ImportWBIPropertiesForSAPChange(IFile file, ImportWBIProperties changeData) {
		super(file, changeData);
	}
	
	public void perform(IProgressMonitor pm) throws Exception {
		if(file.exists()) {
			DOMParser parser = new DOMParser();				
			parser.parse(file.getLocation().toString());				
			Document doc = parser.getDocument();
			
			Element eConnection = (Element)doc.getElementsByTagName("connection").item(0);
			if(eConnection!=null) {
				Element eProperties = (Element)eConnection.getElementsByTagName("properties").item(0);
				Hashtable<String, String> importedProperties = getImportedPropertiesValue();
				for(String importedPropertyName : importedProperties.keySet()) {
					Element eImportedProperty = doc.createElement(importedPropertyName);
					eImportedProperty.setTextContent(importedProperties.get(importedPropertyName));
					eProperties.appendChild(eImportedProperty);
				}
			}
	
			writeXml(file, doc);
		}
	}
	
	private Hashtable<String, String> getImportedPropertiesValue() throws IOException, SAXException {
		Hashtable<String, String> importedProperties = new Hashtable<String, String>();
		
		DOMParser parser = new DOMParser();		
		parser.parse(getChangeData().configFile.getLocation().toString());				
		Document doc = parser.getDocument();
		
		//Added for parsing the password key
		Element configurationInfo = (Element)doc.getElementsByTagNameNS("*", "ConfigurationInfo").item(0);
		Element decryptInstallID = (Element)configurationInfo.getElementsByTagNameNS("*", "installID").item(0);
		String decryptKey = getNodeText(decryptInstallID);
		DecryptPropertyValue decryptPropertyValue = new DecryptPropertyValue();
		
		NodeList nlProperty = doc.getElementsByTagNameNS("*", "property");
		for(int i=0; i<nlProperty.getLength(); i++) {
			Element eProperty = (Element)nlProperty.item(i);
			Element eNode = (Element)eProperty.getElementsByTagNameNS("*", "name").item(0);
			Element eValue = (Element)eProperty.getElementsByTagNameNS("*", "value").item(0);
			String isEncrypted = getNodeText(eProperty.getElementsByTagNameNS("*", "isEncrypted").item(0));
			
			if(eNode!=null && eValue !=null) {
				String name =  getNodeText(eNode);
				String value = getNodeText(eValue);
				
				if(isEncrypted != null && isEncrypted.equalsIgnoreCase("true")) {
					value = decryptPropertyValue.decrypt(value, decryptKey);
				}
				//'Hostname' property 
				if(name.equalsIgnoreCase("Hostname") && value!=null && !value.trim().equals("")) {
					importedProperties.put("gatewayHost", value);
				}
			}
		}
		
		return importedProperties;
	}



}
