/* IBM Confidential
 * OCO Source Materials
 * 5724-L79
 *  Copyright IBM Corporation 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.migration.sap.wbi_to_v610;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;

import org.apache.xerces.parsers.DOMParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.j2ca.migration.MigrationContext;
import com.ibm.j2ca.migration.MigrationException;
import com.ibm.j2ca.migration.util.SearchHelper;
import com.ibm.j2ca.migration.util.Util;
import com.ibm.j2ca.migration.util.XMLUtil;

public class SAPMigrationUtil {
	public static final String COPYRIGHT = " Copyright IBM Corporation 2008."; //$NON-NLS-1$

	public static LinkedHashMap projectTopLevelBOMap = null;
	
	public static LinkedHashMap getSAPProjectAPIMap(MigrationContext context) throws SAXException, IOException, MigrationException, CoreException {
		LinkedHashMap sapProjectAPIMap = new LinkedHashMap();
		projectTopLevelBOMap = new LinkedHashMap();
			
		Collection<IProject> projects = context.getDependentModules();
		for(IProject project : projects) {
			IFile conFile = Util.getWBIConfigFile(project);
			if(conFile.exists()) {
				DOMParser parser = new DOMParser();		
				parser.parse(conFile.getLocation().toString());				
				Document doc = parser.getDocument();
				
				NodeList nlProperty = doc.getElementsByTagNameNS("*", "property");
				for(int i=0; i<nlProperty.getLength(); i++) {
					Element eProperty = (Element)nlProperty.item(i);
					Element eNode = (Element)eProperty.getElementsByTagNameNS("*", "name").item(0);
					Element eValue = (Element)eProperty.getElementsByTagNameNS("*", "value").item(0);
					
					if(eNode!=null && eValue !=null) {
						String name =  XMLUtil.getNodeText(eNode);
						String value = XMLUtil.getNodeText(eValue);
						
						if(name.equalsIgnoreCase("Modules") && value != null) {
						    if(value.equalsIgnoreCase("BAPI")) {
						    	//If modules = BAPI, it could be BAPI, BAPITXN, BAPIRS
						    	ArrayList<IFile> supportedTopLevelBOs = SearchHelper.getSupportedBOs(project);
						    	for(IFile supportedTopLevelBO : supportedTopLevelBOs) {
						    		DOMParser boParser = new DOMParser();		
						    		boParser.parse(supportedTopLevelBO.getLocation().toString());				
									Document boDoc = boParser.getDocument();
									
									NodeList boDefinitions = boDoc.getElementsByTagNameNS("*", "boDefinition");
									if(boDefinitions!=null && boDefinitions.getLength()>0) {
										NodeList appSpecificInfos = ((Element) boDefinitions.item(0)).getElementsByTagNameNS("*", "appSpecificInfo");
										if(appSpecificInfos!=null && appSpecificInfos.getLength()>0) {
											String api = ((Element) appSpecificInfos.item(0)).getTextContent();
											if(api.indexOf("=")>=0) {
												String apiValue = api.substring(api.indexOf("=")+1);
												if(apiValue!=null) {
													if(apiValue.equalsIgnoreCase("bapi_txn"))
													{ value = "BAPITXN"; break; }
													if(apiValue.equalsIgnoreCase("bapi_resultset"))
													{ value = "BAPIRS"; break; }
												}
											}
										}
									}
						    	}
						    }						
							sapProjectAPIMap.put(project, value);
						}
					}
				}
			}
			ArrayList<IFile> toplevelBOs = SearchHelper.getSupportedBOs(project);
			projectTopLevelBOMap.put(project, toplevelBOs);
		}
		return sapProjectAPIMap;
	}
}
